#include "Horde3D.h"
#include "Horde3DUtils.h"
#include <iostream>

#include "Vec3.h"
#include "Game.h"
#include "Airplane.h"

/**
 * constructs an air plane that always flies around
 */
Airplane::Airplane() {
	H3DRes planeRes = h3dAddResource(H3DResTypes::SceneGraph, "models/plane/plane.scene.xml", 0);
	H3DRes particleSysRes = h3dAddResource(H3DResTypes::SceneGraph, "particles/airplaneStream/airplaneStream.scene.xml", 0);
	bool loaded = h3dutLoadResourcesFromDisk(::Values::CONTENT_DIR);
	if(! loaded) std::cout << "could not load airplane ressources" << std::endl;

	_model = h3dAddNodes(H3DRootNode, planeRes);
	//attach smoke to the two airplane engines
	_emitter1 = h3dAddNodes(_model, particleSysRes);
	_emitter2 = h3dAddNodes(_model, particleSysRes);
	h3dSetNodeTransform(_emitter1, 0, 0, -2.2f, 
		90, 0, 0, 
		1, 1, 1);
	h3dSetNodeTransform(_emitter2, 0, 0, 2.2f, 
		90, 0, 0, 
		1, 1, 1);
	h3dSetNodeActivation(_emitter1, true);
	h3dSetNodeActivation(_emitter2, true);

	setPosition(20.0f);
}

Airplane::~Airplane(){
}

/**
 * updates the graphics state and moves the airplane
 * @param timeStep
 */
void Airplane::update(float timeStep) {
	Vec3 speed(2, 0, 0);
	speed *= (::Values::AIRPLANE_SPEED_KM_H / 3.6 * timeStep);
	Vec3 currentPosition = getPosition();
	currentPosition += speed;

	setPosition(currentPosition.x);
	if(currentPosition.x > 400 || currentPosition.x < -400) {
		setPosition(-400);
	}
	//animate particle system
	Game::advanceEmitter(_emitter1, timeStep);
	Game::advanceEmitter(_emitter2, timeStep);
}

/**
 * returns whether the airplane is in proximity to let men jump aboard or not
 * @returns true if the airplane is in proximity to let men jump aboard
 */
bool Airplane::isReadyForJump() {
	Vec3 currentPostion = getPosition();
	//adjust positions according to enemy start positions if necessary
	if(currentPostion.x >= 20 && currentPostion.x <= 32) {
		return true;
	}
	return false;
}

/**
 * returns the current position
 * @return a Vec3 over the current position
 */
Vec3 Airplane::getPosition() {
	float tx, ty, tz;
	float rx, ry, rz;
	float sx, sy, sz;
	h3dGetNodeTransform(_model, &tx, &ty, &tz, &rx, &ry, &rz, &sx, &sy, &sz);
	return Vec3(tx, ty, tz);
}

/**
 * moves the airplane along the x axis
 * @param x the new position in terms of x axis
 */
void Airplane::setPosition(float x) {
	float dropheight = 30.0f; //corresponds to the start position of Enemy
	h3dSetNodeTransform(_model, 
   		x, dropheight, 20, // position // 
	   	0, 0, 0,     // Rotation
		2, 2, 2);    // Scale
}