#pragma once
#include <btBulletDynamicsCommon.h>
#include "Horde3D.h"

#include "CharacterCollisionCallback.h"
#include "Vec3.h"
#include "Physics.h"

//forward declarations
class Physics;

/**
 * This class provides a graphical model simulation for a single bullet
 * that has been shot by the player. After a given period of time or a 
 * given traveling distance the bullet deactivates itself and removes the
 * graphics model. This state can be tested with <pre>isActive()</pre>.
 */
class Bullet {
private:
	H3DNode _model, _bulletProxy;
	Physics* _physics;
	Vec3 _start, _direction, _debug;
	float _activeTime, _distanceToFly;
	float MAX_FLY_TIME_S;
public:
	Bullet(Physics* physics, H3DNode parentNode, H3DRes bulletRes, const float distance);
	~Bullet();
	void update(float timeStep);
	bool isActive();
private:
	Vec3f getNextPosition(float timeStep) const;
};
