#ifndef ENEMY_H_INCLUDED
#define ENEMY_H_INCLUDED

#include "Horde3D.h"
#include "Player.h"
#include "Physics.h"
#include "Game.h"
#include "IShootable.h"

//forward declarations
class Player;
class Game;
class Physics;

/**
 * This class provides an graphics implementation of an enemy which is also subject
 * to the physics world. That is, he collides with other bodies (like the world that he
 * stands on) or the player etc.
 *
 * An instance of Enemy directs to the player's position automatically, so he
 * can damage him (not finished yet)
 * An enemy has a certain "level". The higher the level the better the health and shooting
 * capabilities.
 *
 * If an Enemy has died he may be "reborn" which means his position is reset and he levels up.
 */
class Enemy : public IShootable {
private:
	H3DNode _model;
	H3DRes _modelRes, _texture2, _texture3, _texture4, _fontRes;
	btCapsuleShape* _capsule;
	btDefaultMotionState* _groundMotionState;
	float _capsuleHeight, _bodyRadius, _levelBelowCapsule, _animationTime;
	int _level, LEVEL_TEXTURE_COUNT;
	bool _isReborn, _hasJumped;
	Vec3 _startPosition;
public:
	Enemy(Game* game, Physics* physics, H3DRes font, H3DNode parent, const int id);
	~Enemy();
	void hit();
	void update(H3DNode cam, Player* player, const float timeStep);
	bool reset(const bool doRebirth);
	void setPosition(const Vec3& newPosition);
	Vec3 getPosition() const;
	btRigidBody* getBody() const;
	H3DNode getModel() const;
	bool hasJumped() const;

private:
	bool castRayAndCorrectPosition(Vec3& calculatedCurrentPosition, Vec3& collisionPoint);
	float calcAngleToPlayer(const Vec3 enemyPos, const Vec3 playerPos);
	void displayEnemyInformation(H3DNode cam);
	void Enemy::changeTexture();
};

#endif