#ifndef PHYSICS_H_INCLUDED
#define PHYSICS_H_INCLUDED

#include <vector>
#include <btBulletDynamicsCommon.h>

#include "Vec3.h"
#include "IShootable.h"

//forward declaration
class IShootable;

/**
 * This class provides phyics calculation using the Bullet Physics framework. 
 * The concrete physics models are created in the concerning classes (like World etc.) and
 * MUST submit the phyics body to this this class using either <pre>addShootable()</pre> or
 * <pre>addNonShootable()</pre>
 */
class Physics {
private:
	btAxisSweep3* _broadphase;
	btSequentialImpulseConstraintSolver* _solver;
	btDefaultCollisionConfiguration* _collisionConfiguration;
	btCollisionDispatcher* _dispatcher;
	btDiscreteDynamicsWorld* _dynamicsWorld;
	
	std::vector<IShootable*> _shootableBodies;
	std::vector<btRigidBody*> _nonShootableBodies;
public:
	Physics();
	~Physics();
	void addShootable(IShootable* shootable);
	void addNonShootable(btRigidBody* body);
	void update(const float timeStep);
	bool castRay(const Vec3& from, const Vec3& to, Vec3& collisionPoint, btRigidBody *ignore);
	IShootable* castGunRay(const Vec3& from, const Vec3& to, Vec3& collisionPoint, btRigidBody* ignore);
	btCollisionWorld* getWorld();
	bool removeShootable(IShootable* shootable);
};

#endif