#ifndef PLAYER_H_INCLUDED
#define PLAYER_H_INCLUDED

#include "time.h"
#include <btBulletDynamicsCommon.h>
#include "Horde3D.h"

#include "Vec3.h"
#include "Physics.h"
#include "Enemy.h"
#include "Bullet.h"
#include "IShootable.h"

//forward declarations
class Enemy;
class Physics;
class Bullet;
class Game;

enum Weapon {
   CHAINGUN,
   BOLTER,
   UNSET_WEAPON
 };

class Player : public IShootable {
private:
	btCapsuleShape* _capsule;
	btDefaultMotionState* _groundMotionState;
	H3DNode _cam2, _weaponChainGun, _weaponBolter, _gunfire, _gunFireLight;
	H3DRes _bulletRes;
	Weapon _currentWeaponType;
	float _curWeaponRateOfFire, _animationTime, _weaponRecoilFactor;
	clock_t _timeSinceLastBullet;
	std::vector<Bullet*> _shotBullets;

	float _capsuleHeight, _bodyRadius, _levelBelowCapsule;
	float _rx, _ry;
	bool _debugMode, _wireframeMode;
	int _keys;
	int MOVE_FWD;
	int MOVE_LEFT;
	int MOVE_BACK;
	int MOVE_RIGHT;
	int MOVE_RUN;
	int FIRE_GUN;
	int MOVE_JUMP;
	
public:
	Player(Game* game, Physics* physics, H3DNode camera, const int id);
	~Player();
	void hit();
	void update(float timeStep);
	Vec3 getPosition() const;
	const char* getCurrentWeaponName() const;
	void setKeyboardInput(const int key, bool pushed);
	void setMouseInput(float dX, float dY);
	void setMouseButtonInput(int button, bool action);
	void addReflectionCamera(H3DNode cam2);
private:
	void move(const float zspeed, const float xspeed);
	bool castRayAndCorrectPosition(Vec3& calculatedCurrentPosition, Vec3& collisionPoint);
	IShootable* fireGun(Vec3 currentPosition, float xAngle, float yAngle);
	void setPosition(const Vec3 &newPosition);
	void changeWeapon(Weapon newWeapon);
	int calcShotsPerTimeStep(const float timeStep);
	void removeGunFire();
	void addGunFire(const float timeStep);
};

#endif