#include <iostream>
#include <cstdlib>
#include <cstring>


#include "utOpenGL.h"
#if defined(_MSC_VER) || defined(_WINDOWS_) || defined(_WIN32)
	#include "glfw.h"
#else
	#include <GL/glfw.h>
#endif

#include "Start.h"
#include "Game.h"

const char *caption = "The Game";
const int windowWidth = 800;
const int windowHeight = 600;
static bool fullscreen = false;
static bool running;
static int mx0, my0;

Game *game;

int main(int argc, char* argv[]) {
// Initialize GLFW
	glfwInit();
	if(! setupWindow(windowWidth, windowHeight, fullscreen)) return -1;
	// Initialize application and engine

	//start game logic and graphics
	game = new Game();

	if (! game->init()) {
		// Fake message box
		glfwCloseWindow();
		glfwOpenWindow(800, 16, 8, 8, 8, 8, 24, 8, GLFW_WINDOW);
		glfwSetWindowTitle("Unable to initalize engine - Make sure you have an OpenGL 2.0 compatible graphics card");
		glfwSleep(5);
		
		std::cout << "Unable to initalize engine" << std::endl;
		std::cout << "Make sure you have an OpenGL 2.0 compatible graphics card";
		glfwTerminate();
		return -1;
	}

	game->resize(windowWidth, windowHeight);

	//hide mouse cursor again in favour of the crosshair
	glfwDisable(GLFW_MOUSE_CURSOR);

	float fps = 0.0f;
	double lastTime = glfwGetTime();
	int frames = 0;
	running = true;

	// Game loop
	while(running) {
		// calc timestep
		double currentTime = glfwGetTime();
		float usedTime = currentTime - lastTime; 
		frames++;
		if(frames >= 3){
			fps = frames / usedTime;
			frames = 0;
		}

		lastTime = currentTime;

		// Render
		game->update(usedTime, fps);
		glfwSwapBuffers();
	}

	//show mouse cursor again
	glfwEnable(GLFW_MOUSE_CURSOR);

	// Quit
	game->release();
	delete game;

	try {
		glfwTerminate();
	}
	catch(...) {
		//do nothing here unless you have an idea from where the exception comes
		std::cout << "GLFW returns an error while cleaning up" << std::endl;
	}

	char name[2];
	std::cout << "press enter";
	std::cin.getline (name,2);

	return 0;
}

bool setupWindow(int width, int height, bool fullscreen) {
	// Create OpenGL window
	if(! glfwOpenWindow( width, height, 8, 8, 8, 8, 24, 8, fullscreen ? GLFW_FULLSCREEN : GLFW_WINDOW)) {
		glfwTerminate();
		return false;
	}

	if(! fullscreen) glfwSetWindowTitle( caption);
	
	// Disable vertical synchronization
	glfwSwapInterval(0);

	// Set listeners
	glfwSetWindowCloseCallback(windowCloseListener);
	glfwSetKeyCallback( keyPressListener);
	glfwSetMousePosCallback( mouseMoveListener);
	glfwSetMouseButtonCallback( mouseButtonListener);
	
	return true;
}

int GLFWCALL windowCloseListener() {
	running = false;
	return 0;
}


void GLFWCALL keyPressListener(int key, int action) {
	if(! running) return;

	if( action == GLFW_PRESS) {
		int width = windowWidth;
		int height = windowHeight;
		
		switch (key) {
		case GLFW_KEY_ESC:

			running = false;
			break;
		case GLFW_KEY_SPACE:
			//game->keyPressEvent( key);
			break;
		case GLFW_KEY_F1:

			//game->release();
			//glfwCloseWindow();

			// Toggle fullscreen mode
			fullscreen = !fullscreen;

			if(fullscreen) {
				GLFWvidmode mode;
				glfwGetDesktopMode(&mode);
				
				float aspect = mode.Width / (float)mode.Height;
				if( (int)(aspect * 100) == 133 || (int)(aspect * 100) == 125)	// Standard
				{
					width = 1280;
					height = 1024;
				}
				else if( (int)(aspect * 100) == 160)							// Widescreen
				{
					width = 1280;
					height = 800;
				}
				else															// Unknown
				{
					// Use desktop resolution
					width = mode.Width;
					height = mode.Height;
				}
			}
			
			if(! setupWindow(width, height, fullscreen)) {
				glfwTerminate();
				exit(-1);
			}
			
			game->resize(width, height);
			//lastTime = glfwGetTime();
			break;
		default:
			//game->keyPressEvent(key);
			break;
		}
	}

	if( key >= 0) game->keyStateChange(key, action == GLFW_PRESS);
}


void GLFWCALL mouseMoveListener(int x, int y) {
	if(! running) {
		mx0 = x;
		my0 = y;
		return;
	}

	game->mouseMoveEvent((float)(x - mx0), (float)(my0 - y));
	mx0 = x;
	my0 = y;
}

void GLFWCALL mouseButtonListener(int button, int action) {
	game->mouseButtonEvent(button, action == GLFW_PRESS);
}