#ifndef WORLD_H_INCLUDED
#define WORLD_H_INCLUDED

#include <vector>
#include <btBulletDynamicsCommon.h>
#include "Horde3D.h"

#include "Explosive.h"

//forward declarations
class Physics;
class Game;

/**
 * World provides both a graphical and physical model of a hilly landscape,
 * upon the Player and Enemy objects can move. While gravitation affects all
 * models that are set upon this world, this world prevents by its physical
 * model that other physical entities would fall trough it. Also the lighting
 * environment is setup here as well.
 *
 * This class provides furthermore a set of ambient and kinematic objects that 
 * exist only for graphical representation reasons, like cars, ships etc.
 */
class World {
private:
	btMotionState* motionstate;
	btTriangleMesh* trimesh;
	btBvhTriangleMeshShape* shape;
	btDefaultMotionState* groundMotionState;
	btRigidBody* body;
	H3DNode _vulcanoEmitter, _ship;
	H3DRes _waterplaneRes, _crateRes;
	Physics* _physics;
	Game* _game;
	int _waterplaneMeshMaterial;
	std::vector<Explosive*> _models;
	bool _debugMode;
	float _waterAnimationTime;
public:
	World(Game* game, Physics* physics);
	~World();
	void update(const float timeStep);
	bool testInboundXZ(const Vec3 position);
	void addLight(const char* name, const float px, const float py, const float pz, 
		const float rx, const float ry, const float rz, 
		const int length, const int fov, const int shadowCount, 
		const float r, const float g, const float b);
private:
	void buildGraphicsWorld();
	void buildPhysicsWorld();
	void addPhysicsBox(const Vec3 halfsize, const Vec3 position, const float rotationY);
	H3DNode addModel(H3DRes ressource, Vec3 position, Vec3 rotation, float scale);
};

#endif